/*
 * GpsPostCanvas.java
 *
 * Created on December 2, 2005, 11:35 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package traxx;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.location.Location;
import javax.microedition.location.QualifiedCoordinates;


public class GpsPostCanvas extends Canvas implements CommandListener {
    private static final long MIN_KEY_HOLD_DURATION = 1000;
    
    private static final int BLACK      = 0x00000000;
    private static final int RED        = 0x00FF0000;
    private static final int GREEN      = 0x0000FF00;
    private static final int DARK_GREEN = 0x0000A000;
    
    Command prefCommand;
    Thread keyTimer;
    GpsPost gpsPost;
    int messageID = -1;
//    long keyDownTime;
    
    /** Creates a new instance of GpsPostCanvas */
    public GpsPostCanvas(GpsPost gpsPost)  {
        this.gpsPost = gpsPost;
        prefCommand = new Command("Prefs", Command.SCREEN, 1);
        addCommand(prefCommand);
        setCommandListener(this);
    }
    
    public void paint(Graphics g) {
        Font font = g.getFont();
        int height = font.getHeight() + 1; //1 pixel space between lines
        int y;
        Location location = gpsPost.getLocation();
        QualifiedCoordinates coords;
        
//#if i285
        y = height; //Don't let icons at top cover message text
//#else
//--        y = 0;
//#endif
        
        
        //Message
        g.setColor(BLACK);
        g.fillRect(0, 0, getWidth(), getHeight());
        g.setColor(GREEN);
        
        //This is the first line, which displays either the message or current sensor reading
        if (messageID >= 0) {
            g.drawString(gpsPost.getMessage(messageID), 0, y, g.TOP | g.LEFT);
        } else {
            if (gpsPost.radiationReading != null) {
                g.drawString(("D: " + gpsPost.radiationReading.getGammaDoseRate()
                + " mr/h"), 0, y, g.TOP + g.LEFT);
                g.drawString(("A: " + gpsPost.radiationReading.getAccumulatedDose()
                + " mr"), getWidth()/2, y, g.TOP + g.LEFT);
//            } else if (gpsPost.rae) {
//
            } else {
//                g.drawString("---", 0, y, g.TOP | g.LEFT);
                g.drawString(Integer.toString(gpsPost.pendingSubmissions.size()), 0, y, g.TOP | g.LEFT);
            }
        }
        y += height;
        
        //GPS
        if (location != null) {
            coords = location.getQualifiedCoordinates();
            if (Double.isNaN(coords.getLatitude()) || Double.isNaN(coords.getLongitude())) {
                g.setColor(RED);
                g.drawString("Position unknown", 0, y, g.TOP | g.LEFT);
                g.setColor(GREEN);
            } else {
                String num = Double.toString(Math.abs(coords.getLatitude()));
                String dir = coords.getLatitude() >= 0 ? " N" : " S";
                
                num = GpsPost.trimDecimalPlaces(5, num);
                //And just in case that wasn't enough
                while (font.stringWidth(num) + font.stringWidth(dir) > getWidth()/2 - 3) {
                    num = num.substring(0, num.length()-1);
                }
                
                g.drawString(num + dir, 0, y, g.TOP | g.LEFT);
                
                num = Double.toString(Math.abs(coords.getLongitude()));
                dir = coords.getLongitude() >= 0 ? " E" : " W";
                
                num = GpsPost.trimDecimalPlaces(5, num);
                //And just in case that wasn't enough
                while (font.stringWidth(num) + font.stringWidth(dir) > getWidth()/2) {
                    num = num.substring(0, num.length()-1);
                }
                
                g.drawString(num + dir, getWidth()/2, y, g.TOP | g.LEFT);
            }
            y += height;
            
            if (Float.isNaN(coords.getHorizontalAccuracy())) {
                //Nothing
            } else {
                g.drawString("Within: "
                        + GpsPost.trimDecimalPlaces(1, Float.toString(coords.getHorizontalAccuracy()))
                        + " m", 0, y, g.TOP | g.LEFT);
            }
        } else {
            g.setColor(RED);
            g.drawString("Position unknown", 0, y, g.TOP | g.LEFT);
            g.setColor(GREEN);
            y += height;
        }
        y += height; //For accuracy line
        
        //HTTP Response:
        String response = gpsPost.getNetworkOperation();
        if (response != null) {
            if (gpsPost.getNetworkOperation().indexOf("Error: ") >= 0) {
                g.setColor(RED);
                g.drawString(gpsPost.getNetworkOperation(), 0, y, g.TOP | g.LEFT);
                g.setColor(GREEN);
            } else {
                g.drawString(gpsPost.getNetworkOperation(), 0, y, g.TOP | g.LEFT);
            }
        }
        y += height;
    }
    
    protected void keyPressed(int keyCode) {
        if (KEY_NUM0 <= keyCode && keyCode <= KEY_NUM9) {
            keyCode = keyCode - KEY_NUM0;
//            System.out.println("keyPressed: " + keyCode);
            if (messageID >= 0 && keyCode != messageID) {
                //Ignore second
                return;
            }
            if (keyCode == 0 || gpsPost.getMessage(keyCode) != null) {
//                keyDownTime = System.currentTimeMillis();
                messageID = keyCode;
                repaint();
                
                keyTimer = new Thread("Key timer") {
                    public void run() {
                        try {
                            Thread.currentThread().sleep(MIN_KEY_HOLD_DURATION);
                            
                            gpsPost.getDisplay().vibrate(250);
                            if (messageID >= 0) {
                                if (messageID == 0) {
                                    gpsPost.fireMessage();
                                } else {
                                    gpsPost.fireMessage(messageID);
                                }
                                messageID = -1;
                            }
                            
                            keyTimer = null;
                        } catch (InterruptedException e) {
                            return;
                        }
                    }
                };
                keyTimer.start();
            }
        }
    }
    
    protected void keyReleased(int keyCode) {
        long keyDownDuration;
        if (KEY_NUM0 <= keyCode && keyCode <= KEY_NUM9);
        keyCode = keyCode - KEY_NUM0;
//        System.out.println("keyReleased: " + keyCode);
        if (keyCode == messageID) {
            messageID = -1;
            if (keyTimer != null) {
                keyTimer.interrupt();
            }
            repaint();
        }
    }
    
    public void commandAction(Command command, Displayable displayable) {
        if (command == prefCommand) {
            gpsPost.getDisplay().setCurrent(new PreferencesForm(gpsPost));
        }
    }
}
