/*
 * PreferencesForm.java
 *
 * Created on December 2, 2005, 9:21 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package traxx;

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Spacer;
import javax.microedition.lcdui.TextField;


public class PreferencesForm extends Form implements CommandListener {
    GpsPost gpsPost;
    GpsPostPreferences prefs;
    Command saveCommand = new Command("Save", Command.OK, 1);
    Command cancelCommand = new Command("Cancel", Command.BACK, 1);
    ChoiceGroup postingModeGroup;
    TextField httpServer;
    TextField udpServer;
    TextField userName;
    TextField password;
    ChoiceGroup locationIntervalPostGroup;
    TextField locationPostInterval;
    ChoiceGroup sensorGroup;
    ChoiceGroup sensorIntervalPostGroup;
    TextField sensorPostInterval;
    TextField serialDelimiter;
    ChoiceGroup serialPushGroup;
    TextField[] messages;
    
    /** Creates a new instance of PreferencesForm */
    PreferencesForm(GpsPost gpsPost) {
        super("User Settings");
        this.gpsPost = gpsPost;
        this.prefs = gpsPost.preferences;
        
        postingModeGroup = new ChoiceGroup("Posting mode",
                Choice.EXCLUSIVE,
                new String[] {"HTTP", "UDP"},
                new Image[] {null, null});
        if (prefs.getPostingMode() == GpsPostPreferences.POSTING_MODE_HTTP) {
            postingModeGroup.setSelectedFlags(new boolean[] {true, false});
        } else {
            postingModeGroup.setSelectedFlags(new boolean[] {false, true});
        }
        this.append(postingModeGroup);
        
        httpServer = new TextField("HTTP Server URL", prefs.getHttpUrl(), 256, TextField.URL);
        this.append(httpServer);
        
        udpServer = new TextField("UDP Server URL", prefs.getUdpUrl(), 256, TextField.URL);
        this.append(udpServer);
        
        this.append(new Spacer(1000, 10));
        
        userName = new TextField("User name", prefs.getUserName(), 8, TextField.ANY);
        this.append(userName);
        
        password = new TextField("Password", prefs.getPassword(), 8, TextField.ANY | TextField.PASSWORD);
        this.append(password);
        
        this.append(new Spacer(1000, 10));
        
        locationIntervalPostGroup = new ChoiceGroup("Periodic location",
                Choice.EXCLUSIVE,
                new String[] {"On", "Off"},
                new Image[] {null, null});
        
        if (prefs.isLocationIntervalPostingEnabled()) {
            locationIntervalPostGroup.setSelectedFlags(new boolean[] {true, false});
        } else {
            locationIntervalPostGroup.setSelectedFlags(new boolean[] {false, true});
        }
        this.append(locationIntervalPostGroup);
        
        locationPostInterval = new TextField("Loc interval (s)",
                Integer.toString(prefs.getLocationPostInterval()), 8, TextField.NUMERIC);
        this.append(locationPostInterval);
        
        this.append(new Spacer(1000, 10));
        
        
        sensorGroup = new ChoiceGroup("Sensor",
                Choice.EXCLUSIVE,
                new String[] {"None", "SOR/R", "MultiRae Plus", "Delimited Serial"},
                new Image[] {null, null, null, null});
        
        boolean[] flags = new boolean[4];
        for (int i=0; i < flags.length; i++) {
            flags[i] = false;
        }
        flags[prefs.getSensor()] = true;
        sensorGroup.setSelectedFlags(flags);
        this.append(sensorGroup);
        
        sensorIntervalPostGroup = new ChoiceGroup("Periodic sensor",
                Choice.EXCLUSIVE,
                new String[] {"On", "Off"},
                new Image[] {null, null});
        
        if (prefs.isSensorIntervalPostingEnabled()) {
            sensorIntervalPostGroup.setSelectedFlags(new boolean[] {true, false});
        } else {
            sensorIntervalPostGroup.setSelectedFlags(new boolean[] {false, true});
        }
        this.append(sensorIntervalPostGroup);
        
        sensorPostInterval = new TextField("Sensor interval (s)",
                Integer.toString(prefs.getSensorPostInterval()), 8, TextField.NUMERIC);
        this.append(sensorPostInterval);
        
        this.append(new Spacer(1000, 10));
        
        serialDelimiter = new TextField("Serial delim",
                prefs.getSerialDelimiter(), 10, TextField.ANY);
        this.append(serialDelimiter);
        
        serialPushGroup = new ChoiceGroup("Serial push",
                Choice.EXCLUSIVE,
                new String[] {"On", "Off"},
                new Image[] {null, null});

        if (prefs.isSerialPushEnabled()) {
            serialPushGroup.setSelectedFlags(new boolean[] {true, false});
        } else {
            serialPushGroup.setSelectedFlags(new boolean[] {false, true});
        }
        
        this.append(serialPushGroup);
        
        this.append(new Spacer(1000, 10));
        
        messages = new TextField[prefs.messages.length];
        for (int i=0; i < prefs.messages.length; i++) {
            messages[i] = new TextField("Message " + i, prefs.getMessage(i), 64, TextField.ANY);
            this.append(messages[i]);
        }
        
        addCommand(saveCommand);
        addCommand(cancelCommand);
        setCommandListener(this);
    }
    
    /** Called by the system to indicate that a command has been invoked on
     * a particular displayable.
     * @param command the Command that was invoked
     * @param displayable the Displayable on which the command was invoked
     */
    public void commandAction(Command command, Displayable displayable) {
        // Insert global pre-action code here
        if (command == saveCommand) {
            boolean[] flags = new boolean[sensorGroup.size()];
            prefs.setPostingMode(postingModeGroup.getSelectedIndex());
            prefs.setHttpUrl(httpServer.getString());
            prefs.setUdpUrl(udpServer.getString());
            
            prefs.setUserName(userName.getString());
            prefs.setPassword(password.getString());
            
            prefs.setLocationIntervalPostingEnabled(
                    locationIntervalPostGroup.getSelectedIndex()==0 ? true : false);
            prefs.setLocationPostInterval(Integer.parseInt(locationPostInterval.getString()));
            
            prefs.setSensor(sensorGroup.getSelectedIndex());
            prefs.setSensorIntervalPostingEnabled(
                    sensorIntervalPostGroup.getSelectedIndex()==0 ? true : false);
            prefs.setSensorPostInterval(Integer.parseInt(sensorPostInterval.getString()));
            
            prefs.setSerialDelimiter(serialDelimiter.getString());
            prefs.setSerialPushEnabled(serialPushGroup.getSelectedIndex()==0 ? true : false);
            
            for (int i=0; i < messages.length; i++) {
                prefs.setMessage(i, messages[i].getString());
            }
            
            prefs.save();
//            gpsPost.preferences = prefs;
            gpsPost.restartSensors();
            gpsPost.getDisplay().setCurrent(gpsPost.canvas);
        } else if (command == cancelCommand) {
            gpsPost.getDisplay().setCurrent(gpsPost.canvas);
        }
    }
}
