/*
 * AddressInfo.java
 *
 * Created on November 27, 2005, 7:16 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package javax.microedition.location;

/**
 * The AddressInfo class holds textual address information 
 * about a location. Typically the information is e.g.
 * street address. The information is divided into fields 
 * (e.g. street, postal code, city, etc.). Defined field constants
 * can be used to retrieve field data.
 */
public class AddressInfo {
    public static final int EXTENSION = 1;
    public static final int STREET = 2;
    public static final int POSTAL_CODE = 3;
    public static final int CITY = 4;
    public static final int COUNTY = 5;
    public static final int STATE = 6;
    public static final int COUNTRY = 7;
    public static final int COUNTRY_CODE = 8;
    public static final int DISTRICT = 9;
    public static final int BUILDING_NAME = 10;
    public static final int BUILDING_FLOOR = 11;
    public static final int BUILDING_ROOM = 12;
    public static final int BUILDING_ZONE = 13;
    public static final int CROSSING1 = 14;
    public static final int CROSSING2 = 15;
    public static final int USL = 16;
    public static final int PHONE_NUMBER = 17;
    
    private String[] data = new String[PHONE_NUMBER - 1];
    
    /** Creates a new instance of AddressInfo */
    public AddressInfo() {
    }
    
    /**
     * Returns the value of an address field. If the field is not 
     * available null is returned.
     * @param field field - the ID of the field to be retrieved
     * @return the address field string. If the field is not set, 
     * returns null.
     */
    public String getField(int field) {
        if (field <= 0 || field > PHONE_NUMBER) {
            throw new IllegalArgumentException("Illegal field ID.");
        }
        return data[field - 1];
    }
    
    /**
     * Sets the value of an address field.
     * @param field the ID of the field to be set
     * @param value the new value for the field. null is used to indicate that 
     * the field has no content.
     */
    public void setField(int field, String value) {
        if (field <= 0 || field > PHONE_NUMBER) {
            throw new IllegalArgumentException("Illegal field ID.");
        }        
        data[field - 1] = value;
    }
}
