/*
 * StreamingSORR20.java
 *
 * Created on January 6, 2006, 2:45 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sensor.mgpInstruments;

import sensor.PeriodicEventSource;
import sensor.PeriodicEventSourceImpl;
import util.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

public class StreamingSORR20 extends PeriodicEventSourceImpl {
    
    private InputStream inputStream;
    private BufferedReader br;
    private boolean stop;
    int position;
    ReadThread readThread;
    SORR20Event lastReading;
    
    /**
     * Creates a new instance of StreamingSORR20
     */
    public StreamingSORR20(InputStream is) {
        this.inputStream = is;
        br = new BufferedReader(new InputStreamReader(is));
        readThread = new ReadThread();
        readThread.start();
    }
    
    class ReadThread extends Thread {
        public void run() {
            String data;
            while (!stop) {
                try {
                    data = br.readLine();
                    if (data == null) { //EOS
                        break;
                    }
                    if (data.length() != 46 && data.length() != 47) { //On Windows, is 47 - actually, that's probably just the null
                        continue;
                    }
                    
                    //Otherwise, we think we have a valid sentence
                    lastReading = parseSentence(data);
//                    fireSORR20ListenerNewSORR20Event(lastReading);
//                } catch (IOException ex) {
//                    ex.printStackTrace();
                } catch (Throwable t) {
//                    t.printStackTrace();
                    //Can happen when we yank the comm port from under readLine
                }
            }
            
        }
    }
    
    public SORR20Event parseSentence(String s) throws IOException {
        SORR20Event e = new SORR20Event();
        String temp;
//        System.out.println("S: " + s + "\tLength: " + s.length());
        
        
        if (s.charAt(15) != '0' || s.charAt(39) != '0') {
            throw new IOException("Sentence contained data in unused area.");
        }
        
        //Todo - Checksum
        
        e.setSerialNumber(Integer.parseInt(s.substring(0, 6)));
        
        e.setAccumulatedDose((Float.parseFloat(s.substring(8, 15)))/10);
        
        temp = s.substring(18, 21) + "E" + s.substring(21, 22);
        e.setGammaDoseRate(Float.parseFloat(temp));
        
        e.setAccumulatedBetaDose((Float.parseFloat(s.substring(32, 39)))/10);
        
        temp = s.substring(40, 43) + "E" + s.substring(43, 44);
        e.setBetaDoseRate(Float.parseFloat(temp));
        
//        System.out.println(e);
        return e;
    }
    
    public void stop() {
        stopPolling();
        stop = true;
        readThread.interrupt();
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        if (br != null) {
            //This caused it to hang on br.close when no dosimeter attached.
//            try {
//                br.close();
//            } catch (IOException ex) {
//                ex.printStackTrace();
//            }
        }
    }
    
    /**
     * Utility field holding list of SORR20Listeners.
     */
    private transient java.util.Vector SORR20ListenerList;
    
    /**
     * Registers SORR20Listener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addSORR20Listener(SORR20Listener listener) {
        
        if (SORR20ListenerList == null ) {
            SORR20ListenerList = new java.util.Vector();
        }
        SORR20ListenerList.addElement(listener);
    }
    
    /**
     * Removes SORR20Listener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeSORR20Listener(SORR20Listener listener) {
        
        if (SORR20ListenerList != null ) {
            SORR20ListenerList.removeElement(listener);
        }
    }
    
    /**
     * Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private synchronized void fireSORR20ListenerNewSORR20Event(SORR20Event event) {
        for (int i = 0; i < SORR20ListenerList.size(); i++) {
            ((SORR20Listener)SORR20ListenerList.elementAt(i)).newSORR20Event(event);
        }
    }
    
    protected void fireNewEvent() {
        if (lastReading != null) {
            fireSORR20ListenerNewSORR20Event(lastReading);
            lastReading = null;
        }
    }
}
