/*
 * GpsPost.java
 *
 * Created on December 1, 2005, 3:13 PM
 */

package traxx;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;
import javax.microedition.io.CommConnection;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;
import javax.microedition.io.DatagramConnection;
import javax.microedition.io.HttpConnection;
import javax.microedition.location.QualifiedCoordinates;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.location.Location;
import javax.microedition.location.LocationException;
import javax.microedition.location.LocationListener;
import javax.microedition.location.LocationProvider;
import sensor.delimitedSerial.DelimitedSerialDevice;
import sensor.delimitedSerial.DelimitedSerialEvent;
import sensor.delimitedSerial.DelimitedSerialListener;
import sensor.mgpInstruments.SORR20Event;
import sensor.mgpInstruments.SORR20Listener;
import sensor.mgpInstruments.StreamingSORR20;
import sensor.raeSystems.InstantGasLevelsListener;
import sensor.raeSystems.multiRAEPlus.MultiRAEPlus;

public class GpsPost extends MIDlet implements LocationListener, SORR20Listener, InstantGasLevelsListener, DelimitedSerialListener {
    //Change to false for never-expiring version
    private static final boolean isTimeLimitedDemo = true;
    
    private static final int MAX_QUEUED_REQUESTS = 5;
    private static final int HTTP_TIMEOUT = 5000; //ms
    
    private long httpConnectTime;
    
    /**
     * Holds value of property paused.
     */
    private boolean paused;
    
    private boolean stop;
    
    GpsPostPreferences preferences = new GpsPostPreferences();
    GpsPostCanvas canvas = new GpsPostCanvas(this);
    
    Vector pendingSubmissions = new Vector(); //temporarily package so that Canvas can get to it
    NetworkThread networkThread = new NetworkThread();
    CommConnection commConnection;
    
    private DatagramConnection datagramConnection;
    
    private LocationProvider locationProvider;
    private StreamingSORR20 sorr20;
    private MultiRAEPlus multiRaePlus;
    private DelimitedSerialDevice delimitedSerialDevice;
    
    private String httpResponse;
    private String networkOperation;
    
    private Location location;
    SORR20Event radiationReading;
    
    private void checkExpiration() {
        if (!isTimeLimitedDemo) {
            //Return from function without checking expiration date
            return;
        }
        
        Calendar calendar = Calendar.getInstance();
        calendar.set( Calendar.MONTH, Calendar.JANUARY );
        calendar.set( Calendar.DAY_OF_MONTH, 1 );
        calendar.set( Calendar.YEAR, 2008 );
        
        if (System.currentTimeMillis() > calendar.getTime().getTime()) {
            Alert alert = new Alert("Demo Expired", "The evaluation period for this application has expired.", null, AlertType.ERROR);
            alert.setTimeout(3000);
            getDisplay().setCurrent(alert);
            try {
                Thread.currentThread().sleep(3000);
            } catch (InterruptedException e) { }
            exitMIDlet();
        }
    }
    
    /**
     * Creates a new instance of GpsPost
     */
    public GpsPost() {
        initialize();
        getDisplay().setCurrent(canvas);
        startSensors();
        networkThread.start();
    }
    
    public void startSensors() {
        System.out.println(preferences);
        
        try {
            try {
                if (preferences.isLocationIntervalPostingEnabled()) {
//#if DefaultConfiguration
                    //WTK2.3 emulator won't agree to these criteria
                    locationProvider = LocationProvider.getInstance(null);
//#else
//#                     //But if accuracy criteria not specified, i860 will only use tower
//#                     //if it doesn't get a quick fix.
//#                     javax.microedition.location.Criteria c = new javax.microedition.location.Criteria();
//#                     c.setCostAllowed(false);
//#                     c.setHorizontalAccuracy(10);
//#                     locationProvider = LocationProvider.getInstance(c);
//#endif
                    
                    locationProvider.setLocationListener(this,
                            preferences.getLocationPostInterval(),
                            preferences.getLocationPostInterval(), 1);
                }
            } catch (LocationException ex) {
                ex.printStackTrace();
            }
            
            int sensor = preferences.getSensor();
            
            switch (sensor) {
                case GpsPostPreferences.SENSOR_NONE:
                    break;
                case GpsPostPreferences.SENSOR_SOR_R_20:
                    System.out.println("Port: " + availablePorts()[0]);
                    commConnection = (CommConnection)Connector.open(
                            ("comm:" + availablePorts()[0]
                            + ";baudrate=4800;blocking=on;autocts=off;autorts=off"),
                            Connector.READ_WRITE, false); //No timeouts, please.
                    sorr20 = new StreamingSORR20(commConnection.openInputStream());
                    sorr20.addSORR20Listener(this);
                    sorr20.setEventInterval(preferences.getSensorPostInterval()*1000);
                    if (preferences.isSensorIntervalPostingEnabled()) {
                        sorr20.startPolling();
                    }
                    break;
                case GpsPostPreferences.SENSOR_MULTI_RAE_PLUS:
                    commConnection = (CommConnection)Connector.open(
                            ("comm:" + availablePorts()[0]
                            + ";baudrate=9600;blocking=on;autocts=off;autorts=off"),
                            Connector.READ_WRITE, false); //No timeouts, please.
                    multiRaePlus = new MultiRAEPlus(commConnection.openInputStream(),
                            commConnection.openOutputStream());
                    multiRaePlus.addInstantGasLevelsListener(this);
                    multiRaePlus.setEventInterval(preferences.getSensorPostInterval()*1000);
                    
                    //Powering on a MultiRae can take time, so separate thread
                    new Thread() {
                        public void run() {
                            try {
                                multiRaePlus.setPowerOn(true);
                            } catch (Throwable t) {
                                System.err.println("Error while powering on Multi Rae");
                                t.printStackTrace();
                                return;
                            }
                            if (preferences.isSensorIntervalPostingEnabled()) {
                                multiRaePlus.startPolling();
                            }
                        }
                    }.start();
                    
                    break;
                case GpsPostPreferences.SENSOR_DELIMITED_SERIAL:
                    System.out.println("Port: " + availablePorts()[0]);
                    commConnection = (CommConnection)Connector.open(
                            ("comm:" + availablePorts()[0]
                            + ";baudrate=9600;blocking=on;autocts=off;autorts=off"),
                            Connector.READ_WRITE, false); //No timeouts, please.
                    
                    //Todo - need sleep, or something here to combat:
                    //read failed(00000006)
                    //GetLastError - The handle is invalid.
                    
                    delimitedSerialDevice = new DelimitedSerialDevice(commConnection.openInputStream());
                    
                    //What comes from the prefs panel is a string of space-delimited ASCII codes such as: "10 13"
                    //What DelimitedSerialDevice expects is a String containing the characters correspoding to those codes
                    String delimString = "";
                    try {
                        String[] delims = util.StringSplitCSV.split(preferences.getSerialDelimiter(), " ");
                        for (int i=0; i < delims.length; i++) {
                            delimString += (char) Integer.parseInt(delims[i]);
                        }
                    } catch (Throwable e) {
                        e.printStackTrace();
                        delimString = "\r\n";
                    }
                    delimitedSerialDevice.setDelimiter(delimString);
                    
                    delimitedSerialDevice.setPushEnabled(preferences.isSerialPushEnabled());
                    
                    delimitedSerialDevice.addDelimitedSerialListener(this);
                    delimitedSerialDevice.setEventInterval(preferences.getSensorPostInterval()*1000);
                    if (preferences.isSensorIntervalPostingEnabled()) {
                        delimitedSerialDevice.startPolling();
                    }
                    break;
                default:
                    System.err.println("Default case of sensor switch reached.");
                    
            }
            
        } catch (IOException e) {
            getDisplay().vibrate(1000);
            AlertType.ERROR.playSound(getDisplay());
            networkOperation = "Error: " + e.getMessage();
            if (getDisplay().getCurrent() == canvas && !paused) {
                canvas.repaint();
            }
            e.printStackTrace();
        }
    }
    
    public void stopSensors() {
        if (locationProvider != null) {
            locationProvider.setLocationListener(null, 1, 1, 1);
//            locationProvider.reset();
            locationProvider = null;
        }
        
        if (sorr20 != null) {
            sorr20.removeSORR20Listener(this);
            sorr20.stop();
            sorr20 = null;
        }
        
        if (multiRaePlus != null) {
            multiRaePlus.removeInstantGasLevelsListener(this);
            multiRaePlus.stop();
            multiRaePlus = null;
        }
        
        if (delimitedSerialDevice != null) {
            delimitedSerialDevice.removeDelimitedSerialListener(this);
            delimitedSerialDevice.stop();
            delimitedSerialDevice = null;
        }
        
        if (commConnection != null) {
            try {
                commConnection.close();
//                System.err.println("Comm port closed.");
            } catch (IOException e) {
                System.err.println("Couldn't close comm port: ");
                e.printStackTrace();
            }
        }
        
        if (datagramConnection != null) {
            try {
                datagramConnection.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
//GEN-LINE:MVDFields
    
    /** This method initializes UI of the application.//GEN-BEGIN:MVDMethods
     */
    private void initialize() {
    }//GEN-END:MVDMethods
    
    /**
     * This method should return an instance of the display.
     */
    public Display getDisplay() {//GEN-FIRST:MVDGetDisplay
        return Display.getDisplay(this);
    }//GEN-LAST:MVDGetDisplay
    
    /**
     * This method should exit the midlet.
     */
    public void exitMIDlet() {//GEN-FIRST:MVDExitMidlet
        getDisplay().setCurrent(null);
        destroyApp(true);
        notifyDestroyed();
    }//GEN-LAST:MVDExitMidlet
    
    public String getMessage(int i) {
        return preferences.getMessage(i);
    }
    
    void fireMessage() {
        getDisplay().setCurrent(new MessageEntryForm());
    }
    
    void fireMessage(int messageID) {
        //For confirmation form
        //getDisplay().setCurrent(new MessageConfirmationForm(messageID));
        
        //To skip confirmation
        new MessageSubmission(messageID);
    }
    
    public void startApp() {
        paused = false;
    }
    
    public void pauseApp() {
        paused = true;
    }
    
    public void destroyApp(boolean unconditional) {
        stop = true;
        if (networkThread != null) {
            networkThread.interrupt();
        }
        stopSensors();
    }
    
    /**
     * Getter for property paused.
     * @return Value of property paused.
     */
    public boolean isPaused() {
        return this.paused;
    }
    
    private class NetworkThread extends Thread {
        NetworkThread() {
            super("Network");
        }
        
        public void run() {
            Submission submission = null;
            
            checkExpiration();
            
            while (!stop) {
                try {
                    // If no pending URLs queued up, wait
                    while (pendingSubmissions.isEmpty()) {
                        synchronized(networkThread) {
                            wait();
                        }
                    }
                    
                    synchronized(pendingSubmissions) {
                        if (pendingSubmissions.isEmpty()) {
                            //Preferences save cleared the queue
                            continue;
                        }
                        submission = (Submission) pendingSubmissions.lastElement(); //Send most recent first
                    }
                    
                    if (preferences.getPostingMode() == preferences.POSTING_MODE_HTTP) {
                        submitHttp(submission);
                    } else {
                        submitUdp(submission);
                    }
                    
                } catch (InterruptedException e) {
                    continue;
                }
                
                canvas.repaint();
            }
        }
    }
    
    private void submitHttp(Submission submission) {
        HttpConnection http = null;
        try {
            http = (HttpConnection)Connector.open(buildHttpUrl(submission),
                    Connector.READ_WRITE, true);
            networkOperation = "HTTP GET";
            canvas.repaint();
            http.setRequestMethod(HttpConnection.GET);
            http.setRequestProperty("Connection", "close"); //Todo - needed?
            httpConnectTime = System.currentTimeMillis();
            httpResponse = http.getResponseMessage();
            if (httpResponse.indexOf("OK") >= 0) {
                AlertType.INFO.playSound(getDisplay());
                pendingSubmissions.removeElement(submission);
//                networkOperation = "Server: " + httpResponse;
            } else {
//                AlertType.ERROR.playSound(getDisplay());
                networkOperation = "Error: " + httpResponse;
            }
        } catch (Exception e) {
//            AlertType.ERROR.playSound(getDisplay());
            networkOperation = "Error: " + e.getMessage();
        } finally {
            httpConnectTime = 0;
            if (http != null) {
                try {
                    http.close();
                } catch (IOException ex) {}
            }
        }
    }
    
    private void submitUdp(Submission submission) {
        try {
            if (datagramConnection == null) {
                datagramConnection = (DatagramConnection)
                Connector.open(preferences.getUdpUrl());
            }
            
            networkOperation = "Sending UDP packet";
            canvas.repaint();
            //Url is really the contents of what we want to send
            byte[] bytes = buildUdpBuffer(submission).getBytes();
            Datagram datagram = datagramConnection.newDatagram(bytes, bytes.length);
            datagramConnection.send(datagram);
//            AlertType.INFO.playSound(getDisplay());
            pendingSubmissions.removeElement(submission);
            networkOperation = "Sent UDP Packet";
        } catch (Exception e) {
//            e.printStackTrace();
            AlertType.ERROR.playSound(getDisplay());
            networkOperation = "Error: " + e.getMessage();
            try {
                datagramConnection.close();
            } catch (IOException ioe) {}
            datagramConnection = null;
        }
    }
    
    private String buildHttpUrl(Submission submission) {
        StringBuffer sb = new StringBuffer();
        sb.append(preferences.getHttpUrl());
        sb.append("?");
        sb.append("id=");
        sb.append(preferences.getUserName());
        sb.append("&p=");
        sb.append(preferences.getPassword());
        sb.append("&t=");
        sb.append(System.currentTimeMillis());
//            sb.append("&phone=");
//            sb.append("");
        sb.append(submission.getUrl());
        
        System.out.println(sb.toString());
        return sb.toString();
    }
    
    private String buildUdpBuffer(Submission submission) {
        StringBuffer sb = new StringBuffer();
        sb.append(preferences.getUdpUrl());
        sb.append("?");
        sb.append("id=");
        sb.append(preferences.getUserName());
        sb.append("&p=");
        sb.append(preferences.getPassword());
        sb.append("&t=");
        sb.append(System.currentTimeMillis());
//            sb.append("&phone=");
//            sb.append("");
        sb.append(submission.getUrl());
        
        System.out.println(sb.toString());
        return sb.toString();
    }
    
    private abstract class Submission {
        
        Submission() {
        }
        
        protected void queue() {
            //Keep spotty connection from using too much heap
            synchronized (pendingSubmissions) {
                if (pendingSubmissions.size() >= MAX_QUEUED_REQUESTS) {
                    for (int i=0; i < pendingSubmissions.size(); i++) {
                        if (!(pendingSubmissions.elementAt(i) instanceof MessageSubmission)) {
                            pendingSubmissions.removeElementAt(i);
                            break;
                        }
                    }
                }
                
                pendingSubmissions.addElement(this);
                synchronized (networkThread) {
                    networkThread.notify();
                }
            }
            
            if (httpConnectTime != 0 &&
                    System.currentTimeMillis() - httpConnectTime > HTTP_TIMEOUT) {
                //It's wedged, and would take ~60s to timeout by itself
//                networkThread.interrupt();
//                getDisplay().vibrate(100);
            }
        }
        
        public abstract String getUrl();
    }
    
    private class LocationSubmission extends Submission {
        Location location;
        
        LocationSubmission(Location location) {
            this.location = location;
            queue();
        }
        
        public String getUrl() {
            QualifiedCoordinates coords = location.getQualifiedCoordinates();
            
            StringBuffer sb = new StringBuffer();
            sb.append("&lat=");
            sb.append(trimDecimalPlaces(6, Double.toString(coords.getLatitude())));
            sb.append("&lon=");
            sb.append(trimDecimalPlaces(6, Double.toString(coords.getLongitude())));
//            sb.append("&t=");
//            sb.append(location.getTimestamp());
            sb.append("&sp=");
            sb.append(trimDecimalPlaces(1, Float.toString(location.getSpeed())));
            sb.append("&dir=");
            sb.append(trimDecimalPlaces(1, Float.toString(location.getCourse())));
            sb.append("&herr=");
            sb.append(trimDecimalPlaces(1, Float.toString(coords.getHorizontalAccuracy())));
            
            return sb.toString();
        }
    }
    
    private class MessageSubmission extends Submission {
        int messageID;
        
        MessageSubmission(int messageID) {
            this.messageID = messageID;
            queue();
        }
        
        public String getUrl() {
            return "&status=" + messageID;
        }
    }
    
    private class SORR20Submission extends Submission  {
        SORR20Event reading;
        
        SORR20Submission(SORR20Event reading) {
            this.reading = reading;
            queue();
        }
        
        public String getUrl() {
            return "&radSn=" + reading.getSerialNumber()
            + "&radAccum=" + reading.getAccumulatedDose()
            + "&radGRate=" + reading.getGammaDoseRate()
            + "&radBRate=" + reading.getBetaDoseRate();
        }
    }
    
    private class MultiRaePlusInstantSubmission extends Submission {
        double[] reading;
        
        MultiRaePlusInstantSubmission(double[] reading) {
            this.reading = reading;
            queue();
        }
        
        public String getUrl() {
            StringBuffer sb = new StringBuffer();
            String[] names = multiRaePlus.getSensorNames();
            
            try {
                sb.append("&raeSn=" + multiRaePlus.getSerialNumber());
                if (names != null) {
                    for (int i=0; i < 5; i++) {
                        if (names[i] != null && names[i] != "") {
                            sb.append("&rae" + names[i] + "=" + reading[i]);
                        }
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
            return sb.toString();
        }
    }
    
    private class SerialSubmission extends Submission  {
        String sentence;
        
        SerialSubmission(DelimitedSerialEvent sentence) {
            this.sentence = sentence.getString();
            queue();
        }
        
        public String getUrl() {
            return "&desc=" + sentence;
        }
    }
    
    class MessageConfirmationForm extends Form implements CommandListener {
        Command sendCommand = new Command("Send", Command.OK, 1);
        Command cancelCommand = new Command("Cancel", Command.BACK, 1);
        StringItem message;
        private int messageID;
        
        MessageConfirmationForm(int messageID) {
            super("Confirm send");
            this.messageID = messageID;
            message = new StringItem(null, getMessage(messageID));
            append(message);
            
            addCommand(sendCommand);
            addCommand(cancelCommand);
            setCommandListener(this);
        }
        
        /** Called by the system to indicate that a command has been invoked on
         * a particular displayable.
         * @param command the Command that was invoked
         * @param displayable the Displayable on which the command was invoked
         */
        public void commandAction(Command command, Displayable displayable) {
            // Insert global pre-action code here
            if (command == sendCommand) {
                new MessageSubmission(messageID);
            } else if (command == cancelCommand) {
                //Command listener, too?
            }
            getDisplay().setCurrent(canvas);
        }
    }
    
    class MessageEntryForm extends Form implements CommandListener {
        Command sendCommand = new Command("Send", Command.OK, 1);
        Command cancelCommand = new Command("Cancel", Command.BACK, 1);
        TextField textField;
        
        MessageEntryForm() {
            super("Enter message");
            textField = new TextField("Message", "", 128, TextField.ANY);
            append(textField);
            
            addCommand(sendCommand);
            addCommand(cancelCommand);
            setCommandListener(this);
        }
        
        /** Called by the system to indicate that a command has been invoked on
         * a particular displayable.
         * @param command the Command that was invoked
         * @param displayable the Displayable on which the command was invoked
         */
        public void commandAction(Command command, Displayable displayable) {
            // Insert global pre-action code here
            if (command == sendCommand) {
                new SerialSubmission(new DelimitedSerialEvent(textField.getString()));
            } else if (command == cancelCommand) {
                //Command listener, too?
            }
            getDisplay().setCurrent(canvas);
        }
    }    
    
    public void locationUpdated(LocationProvider provider, Location location) {
        this.location = location;
        if (getDisplay().getCurrent() == canvas && !paused) {
            canvas.repaint();
        }
        new LocationSubmission(location);
    }
    
    public void providerStateChanged(LocationProvider provider, int newState) {
    }
    
    public void newSORR20Event(SORR20Event e) {
        radiationReading = e;
        if (getDisplay().getCurrent() == canvas && !paused) {
            canvas.repaint();
        }
        new SORR20Submission(e);
    }
    
    public void newInstantGasLevels(double[] levels) {
//        radiationReading = e;
//        if (getDisplay().getCurrent() == canvas && !paused) {
//            canvas.repaint();
//        }
        new MultiRaePlusInstantSubmission(levels);
    }
    
    public void newDelimitedSerialEvent(DelimitedSerialEvent e) {
        new SerialSubmission(e);
    }
    
    public String getNetworkOperation() {
        return networkOperation;
    }
    
    public static String trimDecimalPlaces(int numPlaces, String toTrim) {
        String copy = toTrim.concat(""); //Copy
        if (copy.indexOf("NaN") >= 0) {
            //This shouldn't happen.
            return copy;
        }
        int pastPoint = copy.indexOf('.') + 1;
        while (copy.length()-pastPoint < numPlaces) {
            copy = copy.concat("0");
        }
        return copy.substring(0, pastPoint + numPlaces);
    }
    
    Location getLocation() {
        return location;
    }
    
    public void restartSensors() {
        new Thread() {
            public void run() {
                try {
                    networkOperation = "Reconfiguring";
                    synchronized (pendingSubmissions) {
                        pendingSubmissions.removeAllElements();
                    }
                    location = null;
                    radiationReading = null;
                    //rae
                    canvas.repaint();
                    stopSensors();
                    startSensors();
                    networkOperation = "";
                    location = null;
                    radiationReading = null;
                    //rae
                    canvas.repaint();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.start();
    }
    
    private String[] availablePorts() {
        
        Vector vect = new Vector();
        String ports = System.getProperty("microedition.commports");
        int start = 0;
        int end = 0;
        while ((end = ports.indexOf(',', start)) != -1) {
            vect.addElement(ports.substring(start, end).trim());
            start = end + 1;
            ports = ports.substring(start);
        }
        vect.addElement(ports);
        String[] retVal = new String[vect.size()];
        for (int i = 0; i < retVal.length; i++) {
            retVal[i] = (String) vect.elementAt(i);
        }
        return retVal;
    }
}
