/*
 * UDPReceiver.java
 *
 * Created on October 11, 2004, 11:33 AM
 */

import java.io.*;
import java.net.*;
import java.util.*;

public class UDPReceiver {
//    public static final String DEFAULT_HOST_URL = "http://www.autonomechs.com/cgi-bin/traxx.pl?";
    public static final String DEFAULT_HOST_URL = "http://www.autonomechs.com/cgi-bin/prism.pl?";
    //&id=666&lat=32.12345&lon=-75.12345&time=123456.78 //sample of rest of URL
    public static final int DEFAULT_PORT = 1721;
    public static final int DEFAULT_ID = 10;
    private static final int MAX_PACKET_SIZE = 2048;
    
    private String hostUrl;
    private int udpPort;
    private int id;
    private DatagramSocket socket;
    private boolean quit;
    private String urlString;
    private String urlStringRad;
    
    /** Creates a new instance of UDPReceiver */
    public UDPReceiver(String hostUrl, int udpPort, int id) {
        this.hostUrl = hostUrl;
        this.udpPort = udpPort;
        this.id = id;
        try {
            socket = new DatagramSocket(udpPort);
        } catch (SocketException e) {
            e.printStackTrace();
        }
    }
    
    class PacketReceiver extends Thread {
        public void run() {
            byte[] buffer = new byte[MAX_PACKET_SIZE];
            DatagramPacket packet;
            String packetData;
            int sentenceIndex = 0;
            int leftTag = -1;
            int rightTag = -1;
            int radFlag;        //Rad Flag
            String[] packetTokens;
            GPRMCEvent gprmcEvent;
//            String urlString;
//            String urlStringRad;
            URL url;
            URLConnection urlConnection;
            
            while (!quit) {
                try {
                    packet = new DatagramPacket(buffer, buffer.length);
                    socket.receive(packet);
                    
                    packetData = new String(packet.getData(),
                            0, packet.getLength(), "US-ASCII");
                    
                    System.out.print("Packet - ");
                    
                    sentenceIndex = packetData.indexOf("$GPRMC");
                    radFlag = 0;
                    if (sentenceIndex <= 0) {
                        //Didn't contain an NMEA GPRMC sentence
                        System.out.println("no GPRMC sentence");
                        System.out.println(packetData);
                        //Look for Radiation RD tag and the start/stop <> markers
                        if (packetData.indexOf("RD") == 0){
                            leftTag = packetData.indexOf("<");
                            rightTag = packetData.indexOf(">");
                            //System.out.println ("Lt-Rt " + Integer.toString(leftTag) + Integer.toString(rightTag));
                            if ((leftTag>0)&&(rightTag>leftTag+1)){
                                String radiationLevel = packetData.substring(leftTag+1, (rightTag));
                                System.out.println (radiationLevel);
                                radFlag = 1;
                                urlStringRad = hostUrl + "&id=" + id + "&com1=" + radiationLevel;
                            }
                        }
                        else continue;
//                        saveDataToFile(packetData);
                    }
                    else{                    
                    gprmcEvent = new GPRMCEvent(packetData.substring(
                            sentenceIndex, packetData.length()-1));
                            System.out.println(gprmcEvent);
                    
                    if (!gprmcEvent.signalAcquired) {
                        System.out.println("signal not acquired");
                        continue;
                    }
                    
                    //System.out.println(
                    //        "lat=" + gprmcEvent.latitudeDecimalDegrees
                    //        + " lon=" + gprmcEvent.longitudeDecimalDegrees
                    //        + " time=" + gprmcEvent.dateTime);
                    
                    
                    urlString = hostUrl
                            + "&id=" + id
                            + "&lat=" + gprmcEvent.latitudeDecimalDegrees
                            + "&lon=" + gprmcEvent.longitudeDecimalDegrees
                            + "&mil=" + gprmcEvent.speedOverGround
                            + "&health=" + gprmcEvent.headingOverGround
                            + "&com2=" + gprmcEvent.signalAcquired
                            + "&com3=" + id + "_SBT_" + gprmcEvent.speedOverGround +"_"+gprmcEvent.headingOverGround + "_"+gprmcEvent.timeDbl
                            + "&time=" + gprmcEvent.timeDbl;
                    
                    //System.out.println(urlString);
                    if(radFlag==0){
                        saveDataToFile(id
                                + "," + gprmcEvent.latitudeDecimalDegrees
                                + "," + gprmcEvent.longitudeDecimalDegrees
                                + "," + gprmcEvent.timeDbl
                                + "," + gprmcEvent.speedOverGround
                                + "," + gprmcEvent.headingOverGround
                                + "," + gprmcEvent.signalAcquired
                                + "," + id + "_SBT_" + gprmcEvent.speedOverGround +"_"+gprmcEvent.headingOverGround + "_"+gprmcEvent.timeDbl);

                        }
                    }
                    if (radFlag==0) url = new URL(urlString);
                    else url = new URL(urlStringRad);
                    urlConnection = url.openConnection();
                    
                                        DataInputStream in = new
                                               DataInputStream(urlConnection.getInputStream());
                    
                                        String line;
                                       // System.out.println("Result:");
                                        while ((line = in.readLine()) != null) {
                                            //System.out.println(" " + line);
                                        }
                    
                } catch (IOException e) {
                    if (!quit) { //Don't care about exception from socket.close
                        e.printStackTrace();
                    }
                }
            }
        }
    }
    
    public void startReadingPackets() {
        System.out.println("Waiting for UDP packets");
        PacketReceiver packetReceiver = new PacketReceiver();
        packetReceiver.start();
    }
    
    public void saveDataToFile (String sData)
    {
// Save the data in a File.................
        Date today = new Date();
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(today);
        System.out.println("Saving Data");
        File inFile = new File (calendar.get(Calendar.DAY_OF_MONTH) + "-" + (calendar.get(Calendar.MONTH)+1) + "-" + calendar.get(Calendar.YEAR) + "_" + Integer.toString(DEFAULT_ID) + ".txt");
        try 
        {
                FileWriter in = new FileWriter(inFile, true);

                in.write(sData + "\r\n");
                in.close();
        }
        catch (IOException ioe)
        {
                System.err.println(ioe);
        }
    }
    
    public void stopReadingPackets() {
        if (!quit) {
            System.out.println("Quitting");
        }
        this.quit = true;
        if (socket != null) {
            socket.close();
        }
    }
    
    public void finalize() {
        stopReadingPackets();
    }
    
    public static void main(String args[]) {
        int key;
        UDPReceiver udpReceiver;
        if (args.length == 3) {
            udpReceiver = new UDPReceiver(args[0], Integer.parseInt(args[1]),
                    Integer.parseInt(args[2]));
        } else {
            udpReceiver = new UDPReceiver(
                    DEFAULT_HOST_URL, DEFAULT_PORT, DEFAULT_ID);
        }
        System.out.println("Press Enter to quit");
        udpReceiver.startReadingPackets();
        
        //Wait for Enter
        do {
            try {
                key = System.in.read();
            } catch (IOException e) {
                break;
            }
        } while (key != 10 && key != 13);
        
        udpReceiver.stopReadingPackets();
    }
}
